# settings.py
#
# Copyright 2020-2025 Fabio Comuni, et al.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from typing import Self, Optional
from enum import IntEnum

from gi.repository import Gio
from gi.repository import Gtk
from dateutil import tz


class UnitializedError(Exception):
    pass


# in secs
CACHE_NOCACHE = 0
CACHE_FIVE_MINUTES = 5 * 60
CACHE_ONE_HOUR = 60 * 60
CACHE_ONE_DAY = 24 * CACHE_ONE_HOUR
CACHE_ONE_WEEK = 7 * CACHE_ONE_DAY


class CacheDuration(IntEnum):
    NO_CACHE = 0
    FIVE_MINUTES = 1
    ONE_HOUR = 2
    ONE_DAY = 3
    ONE_WEEK = 4

    @classmethod
    def as_model(cls) -> Gtk.StringList:
        labels = [n.name.replace("_", " ").title() for n in cls]
        return Gtk.StringList.new(labels)

    @classmethod
    def get_duration(cls, enum_value):
        return [
            CACHE_NOCACHE,
            CACHE_FIVE_MINUTES,
            CACHE_ONE_HOUR,
            CACHE_ONE_DAY,
            CACHE_ONE_WEEK,
        ][enum_value]


class Settings(Gio.Settings):
    __gtype_name__ = "ConfySettings"
    _instance: Optional[Self] = None

    def __init__(self):
        """
            Init settings
        """
        Gio.Settings.__init__(self)

    @classmethod
    def instance(kls) -> Self:
        if kls._instance is None:
            raise UnitializedError("No instance. Settings has not intialized.")
        return kls._instance

    @classmethod
    def init(kls, app_id: str) -> Self:
        """
            Initialize and return a new Settings object
        """
        settings = Gio.Settings.new(app_id)
        settings.__class__ = kls
        kls._instance = settings
        return settings

    def get_list_cache(self) -> CacheDuration:
        return CacheDuration.get_duration(self.get_int('list-cache'))

    def get_event_cache(self) -> CacheDuration:
        return CacheDuration.get_duration(self.get_int('event-cache'))

    def get_timezone(self):
        return tz.tzlocal()

    def get_size(self) -> tuple[int, int]:
        return (self.get_int('window-width'), self.get_int('window-height'))

    def get_maximized(self) -> bool:
        return self.get_boolean('window-maximized')

    def set_size(self, size: tuple[int, int]):
        self.set_int('window-width', size[0])
        self.set_int('window-height', size[1])

    def set_maximized(self, maximized: bool):
        self.set_boolean('window-maximized', maximized)




# widgets/author.py
#
# Copyright 2020-2024 Fabio Comuni, et al.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from gi.repository import Gio
from gi.repository import Gtk
from gi.repository import Gdk
from gi.repository import Adw

from .. import models
from .. import local
from ..utils import clean_markup


class Author(Gtk.Box):
    __gtype_name__ = "ConfyAuthor"

    def __init__(self, person: models.Person):
        super().__init__(spacing=8)
        self.person = person

        name = clean_markup(person.name)

        self.avatar = avatar = Adw.Avatar.new(24, name, True)
        self.append(avatar)

        self.label = label = Gtk.Label(label=name)
        label.props.halign = Gtk.Align.START
        self.append(label)

        print(person.thumbnail)
        if person.thumbnail:
            self._f = local.get_image_async(person.thumbnail, self._set_image)

    def _set_image(self, fetcher, localfile):
        self._f = None
        self.avatar.set_custom_image(
            Gdk.Texture.new_from_file(
                Gio.File.new_for_path(localfile)
            )
        )



# widgets/eventdialog.py
#
# Copyright 2020-2023 Fabio Comuni, et al.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from gi.repository import Adw
from gi.repository import GObject
from gi.repository import Gtk


@Gtk.Template(resource_path="/net/kirgroup/confy/widgets/eventeditdialog.ui")
class EventEditDialog(Adw.Dialog):
    __gtype_name__ = "ConfyEventEditDialog"

    CANCEL = Gtk.ResponseType.CANCEL
    OK = Gtk.ResponseType.OK
    DELETE = 1

    _can_save = False
    _can_delete = False
    _can_cancel = False

    @GObject.Property(type=bool, default=False)
    def can_save(self):
        return self._can_save

    @can_save.setter
    def can_save_setter(self, value):
        self._can_save = value

    @GObject.Property(type=bool, default=False)
    def can_delete(self):
        return self._can_delete

    @can_delete.setter
    def can_delete_setter(self, value):
        self._can_delete = value

    @GObject.Property(type=bool, default=False)
    def can_cancel(self):
        return self._can_cancel

    @can_cancel.setter
    def can_cancel_setter(self, value):
        self._can_cancel = value

    @GObject.Signal
    def save(self):
        pass

    @GObject.Signal
    def cancel(self):
        pass

    @GObject.Signal
    def delete(self):
        pass

    entry_url = Gtk.Template.Child()  # type: ignore
    entry_title = Gtk.Template.Child()  # type: ignore
    entry_start = Gtk.Template.Child()  # type: ignore
    entry_end = Gtk.Template.Child()  # type: ignore
    entry_icon = Gtk.Template.Child()  # type: ignore

    def __init__(self, parent, conf, **kwargs):
        self._err = set()
        self.conf = conf
        super().__init__(**kwargs)

        self.setup_ui()

    def get_conf(self):
        return self.conf

    def _do_ok(self):
        self.conf.url = self.entry_url.get_text()
        self.conf.title = self.entry_title.get_text()
        self.conf.start = self.entry_start.get_date()
        self.conf.end = self.entry_end.get_date()

        if self.conf.metadata is None:
            self.conf.metadata = {}

        if self.entry_icon.get_text() != "":
            self.conf.metadata['icon'] = self.entry_icon.get_text()
        elif 'icon' in self.conf.metadata:
            del self.conf.metadata['icon']

    @Gtk.Template.Callback()
    def save_event(self, entry):
        self._do_ok()
        self.emit("save")

    @Gtk.Template.Callback()
    def cancel_event(self, entry):
        self.emit("cancel")

    @Gtk.Template.Callback()
    def delete_event(self, entry):
        self.emit("delete")

    @Gtk.Template.Callback()
    def validate_entry_required(self, entry):
        if len(entry.get_text()) > 0:
            entry.remove_css_class("error")
            if entry in self._err:
                self._err.remove(entry)
        else:
            entry.add_css_class("error")
            self._err.add(entry)
        self._update_ok()

    @Gtk.Template.Callback()
    def validate_date_order(self, entry):
        if self.entry_start.get_date() is None or self.entry_end.get_date() is None:
            entry.add_css_class("error")
            self._err.add(entry)
        elif self.entry_start.get_date() > self.entry_end.get_date():
            entry.add_css_class("error")
            self._err.add(entry)
        else:
            for e in (self.entry_start, self.entry_end):
                e.remove_css_class("error")
                if e in self._err:
                    self._err.remove(e)
        self._update_ok()

    def _update_ok(self):
        self.props.can_save = len(self._err) == 0

    def setup_ui(self):
        self.entry_url.set_text(self.conf.url)
        self.entry_title.set_text(self.conf.title)
        self.entry_start.set_date(self.conf.start)
        self.entry_end.set_date(self.conf.end)

        if self.conf.metadata is not None:
            self.entry_icon.set_text(self.conf.metadata.get('icon', ''))



# main.py
#
# Copyright 2020-2024 Fabio Comuni, et al.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
from gettext import gettext as _

import gi
gi.require_version('Gtk', '4.0')
gi.require_version('Adw', '1')
gi.require_version("Notify", "0.7")

from gi.repository import Gtk, Gdk, Gio, GLib, GObject
from gi.repository import Adw
from gi.repository import Notify

from . import local
from .config import APP_NAME, APP_ID, APP_VERSION  # type:ignore # config is generated at compile time
from .preferences import PreferencesWindow
from .openwindow import OpenWindow
from .mainwindow import MainWindow


class Application(Gtk.Application):
    @GObject.Signal
    def tick(self):
        """Event fired every minute

        Used to check for notifications to send and to update events row background
        """
        pass

    def __init__(self):
        super().__init__(application_id=APP_ID,
                         flags=Gio.ApplicationFlags.FLAGS_NONE)
        GLib.set_application_name(APP_NAME)
        GLib.set_prgname(APP_ID)

        display = Gdk.Display.get_default()

        # resource icons in icon theme path
        icon_theme = Gtk.IconTheme.get_for_display(display)
        icon_theme.add_resource_path("/net/kirgroup/confy/icons")

        # custom css
        css_provider = Gtk.CssProvider()
        css_provider.load_from_resource("/net/kirgroup/confy/confy.css")
        Gtk.StyleContext.add_provider_for_display(
            display,
            css_provider,
            Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)

    def run(self, argv):
        self.connect("startup", self.on_startup)
        self.connect("activate", self.on_activate)
        self.connect("shutdown", self.on_shutdown)
        return super().run(argv)

    def on_startup(self, *_):
        Adw.init()
        Notify.init(APP_NAME)
        local.init(APP_ID)

        self._add_action('quit', None, ["<Primary>q",], lambda *args: self.quit())
        self._add_action('shortcuts', None, ["<Primary>question"], self.show_shortcuts)
        self._add_action('preferences', None, ["<Primary>comma",], self.show_preferences)
        self._add_action('about', None, None, self.show_about)

        GLib.timeout_add_seconds(30, self._do_tick)

    def _add_action(self, name, parameter_type, accels, activate_cbk):
        action = Gio.SimpleAction.new(name, parameter_type)
        action.connect("activate", activate_cbk)
        self.add_action(action)
        if accels is not None:
            self.set_accels_for_action(F"app.{name}", accels)
        return action

    def _do_tick(self):
        self.emit("tick")
        return True

    def open_conf(self, conf):
        win = self.props.active_window
        if isinstance(win, OpenWindow):
            win.close()
            win = self.props.active_window
        win.load_conf(conf)
        win.present()

    # def start_window(self):
    #    a_win = self.props.active_window
    #    win = OpenWindow(application=self)
    #    if isinstance(a_win, MainWindow):
    #        win.connect("show", lambda *_: a_win.destroy())
    #    win.present()

    def on_activate(self, *_):
        win = self.props.active_window
        if not win:
            win = MainWindow(application=self)
        win.present()

    def on_shutdown(self, *_):
        local.close()

    def show_shortcuts(self, *_):
        b = Gtk.Builder.new_from_resource("/net/kirgroup/confy/shortcuts.ui")
        w = b.get_object("shortcuts-win")
        w.set_transient_for(self.props.active_window)
        w.props.view_name = None
        w.show()

    def show_preferences(self, *args):
        pref = PreferencesWindow(transient_for=self.props.active_window)
        pref.show()

    def show_about(self, *args):
        Adw.AboutWindow(
            application_name=APP_NAME,
            application_icon=APP_ID,
            version=APP_VERSION,
            copyright="© 2020-2023 Fabio Comuni, et al.",
            website="https://confy.kirgroup.net/",
            issue_url="https://todo.sr.ht/~fabrixxm/confy",
            developers=["Fabio Comuni", "Evangelos Ribeiro Tzaras", "Sebastian Crane"],
            # Translators: about dialog text; this string should be replaced by a text crediting yourselves and your translation team, or should be left empty. Do not translate literally!
            translator_credits=_("translator-credits"),
            comments=_("Conference schedules viewer"),
            license_type=Gtk.License.GPL_3_0
        ).show()


def main(version):
    app = Application()
    return app.run(sys.argv)



# widgets/inputdialog.py
#
# Copyright 2020-2023 Fabio Comuni, et al.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from gettext import gettext as _

from gi.repository import Gtk
from gi.repository import Adw


class InputDialog(Adw.MessageDialog):
    CANCEL = "cancel"
    OK = "ok"

    def __init__(self, parent, **kwargs):
        super().__init__(transient_for=parent)
        self.props.heading = _("Add Event from URL")
        self.props.body = _("Enter URL to event schedule to add it to the list.\nConfy supports events in Pentabarf (.xml) or iCal (.ics) format.")

        self.add_response(self.CANCEL, _("Cancel"))
        self.add_response(self.OK, _("Add"))
        self.set_response_appearance(self.OK, Adw.ResponseAppearance.SUGGESTED)
        self.set_default_response(self.OK)
        self.set_close_response(self.CANCEL)

        self.props.extra_child = Gtk.Entry(
            activates_default=True,
            editable=True,
        )

    def get_text(self):
        return self.props.extra_child.get_text()



def up(cursor):
    cursor.execute("DROP TABLE IF EXISTS event_person")
    cursor.execute("""CREATE TABLE event_person (
        event_id TEXT(50),
        person_id TEXT(50),
        FOREIGN KEY(event_id) REFERENCES events(id),
        FOREIGN KEY(person_id) REFERENCES persons(id),
        UNIQUE(event_id, person_id))""")


def down(cursor):
    ...



# models.py
#
# Copyright 2020-2025 Fabio Comuni, et al.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from typing import Optional, Any, Union
import datetime

from gi.repository import GObject, Gtk
from dateutil.tz import UTC

from . import local
from .settings import Settings


class ObjectNotFound(Exception):
    pass


DayOrDateOrStr = Optional[Union["Day", str, datetime.date]]
RoomOrName = Optional[Union["Room", str]]
TrackOrName = Optional[Union["Track", str]]
EventOrId = Optional[Union["Track", str]]


# models
def _build_filter(
        table: str = "",
        *,
        day: DayOrDateOrStr = None,
        room: RoomOrName = None,
        track: TrackOrName = None,
        event: EventOrId = None,
        **kwargs):
    filterdata = []
    wherestm = []

    if table != "":
        table = f"{table}."

    def __f(col, value):
        if value is None:
            wherestm.append(f"{table}{col} is null")
        else:
            wherestm.append(f"{table}{col} = ?")
            filterdata.append(value)

    match day:
        case str():
            __f("date", day)
        case datetime.date():
            __f("date", day)
        case Day():
            __f("date", day.date)

    match room:
        case str():
            __f("room", room)
        case Room():
            __f("room", room.name)

    match track:
        case str():
            __f("track", track)
        case Track():
            __f("track", track.name)

    match event:
        case str():
            __f("id", event)
        case Event():
            __f("id", event.id)

    for k, v in kwargs.items():
        __f(k, v)

    return (filterdata, wherestm)


class Conference:
    """A conference in menu files (ggmenu.json and usermenu.json)"""

    # was added by user (true) or come from giggity (false)?
    user: bool = False

    id: str = ""
    version: str = ""
    url: str = ""
    title: str = ""
    timezone: str = "UTC"   # TODO: should default to current tz?
    start: datetime.date = datetime.date.today()
    end: datetime.date = datetime.date.today()

    metadata: dict[str, Any] = {}

    def _init(self, obj):
        self.metadata = {}
        self.user = False
        for k, v in obj.items():
            setattr(self, k, v)

        self.start = datetime.date(*[int(s) for s in self.start.split("-")])
        self.end = datetime.date(*[int(s) for s in self.end.split("-")])

        return self

    def get_map_links(self):
        maps = [link for link in self.metadata.get('links', []) if link['title'] == "Map"]
        return maps

    def get_map_files(self, cbk):
        r = []
        for link in self.get_map_links():
            r.append(local.get_image_async(link['url'], cbk))
        return r

    def get_logo_file(self, cbk):
        logourl = self.metadata.get('icon', None)
        return local.get_image_async(logourl, cbk)

    def __repr__(self):
        return f"<{self.__class__.__name__} {self.__dict__}>"

    @classmethod
    def all(cls):
        # from giggity menu
        return [cls()._init(s) for s in local.get_menu()]

    @classmethod
    def by_url(cls, url):
        ss = [s for s in local.get_menu() if url in s['url']]
        if len(ss) == 0:
            return None
        return cls()._init(ss[0])

    def has_cache(self):
        """Check if there is local db for this conference"""
        return local.existsdb(self.url)

    def get_meta(self):
        """Return metadata from db. This will close any opened db connection."""
        local.close()
        local.opendb(self.url, False)
        return Meta()

    def to_json(self):
        """Export conference meta as json for user menu"""
        d = self.__dict__.copy()
        for k in d:
            if isinstance(d[k], (datetime.date, datetime.datetime)):
                d[k] = d[k].strftime("%Y-%m-%d")
        return d


class Meta:
    """Conference metadata from "meta" table in db"""
    def __init__(self):
        _c = local.getDb().execute("SELECT key, value FROM meta")
        _d = [(row['key'], self._fmtd_value(row['key'], row['value'])) for row in _c]
        self._data = {'last_update': 0}
        self._data.update(dict(_d))

    def _fmtd_value(self, k, v):
        if k == "start" or k == "end":
            if v is not None and v != "":
                # drop any time info and keep only date
                v = v.split(" ")[0]
                v = v.split("T")[0]
                v = datetime.date(*[int(s) for s in v.split("-")])
            else:
                v = None
        if k == "last_update":
            v = float(v)
        return v

    def __getattr__(self, name):
        if name.startswith("_"):
            return super().__getattr__(name)
        return self._data.get(name)

    def __setattr__(self, name, value):
        if name.startswith("_"):
            return super().__setattr__(name, value)
        self._data[name] = value

    def __enter__(self):
        return self

    def __exit__(self, *args):
        self.save()

    def __contains__(self, name):
        return name in self._data

    def __repr__(self):
        return f"<{self.__class__.__name__} {self.__dict__}>"

    def get(self, name, default=None):
        #print("Meta.get", name, "->", self._data.get(name, default))
        return self._data.get(name, default)

    def set(self, name, value):
        #print("Meta.set", name, value)
        self._data[name] = value

    def save(self):
        db = local.getDb()
        if db is None:
            local.opendb(self.url, False)
            db = local.getDb()
        print(self._data.items())
        db.executemany("INSERT OR REPLACE INTO meta (key, value) VALUES (?,?)", self._data.items())
        db.commit()

    def to_json(self):
        """Export conference meta as json for user menu"""
        return {
            "url": self.url,
            "title": self.title if self.title else "",
            "start": self.start.strftime("%Y-%m-%d"),
            "end": self.end.strftime("%Y-%m-%d"),
            "user": True
        }


class Day:
    def _init(self, data):
        self.date = data['date']
        return self

    def __repr__(self):
        return "<{} {!r}>".format(self.__class__.__name__, self.date)

    def __str__(self):
        return self.date.strftime("%x")

    @classmethod
    def all(cls):
        for row in local.getDb().execute("SELECT date FROM events GROUP BY date"):
            yield cls()._init(row)

    @classmethod
    def count(cls):
        for row in local.getDb().execute("SELECT count(DISTINCT date) FROM events"):
            return row[0]

    @classmethod
    def filter(cls, room=None, track=None, event=None):
        filterdata, wherestm = _build_filter(room=room, track=track, event=event)

        query = """SELECT date FROM events
                    WHERE {}
                    GROUP BY date""".format(" AND ".join(wherestm))

        for row in local.getDb().execute(query, filterdata):
            yield cls()._init(row)


class Room:
    def _init(self, data):
        self.name = data['room']
        return self

    def __repr__(self):
        return "<{} {!r}>".format(self.__class__.__name__, self.name)

    def __str__(self):
        return self.name or "- no room -"

    @classmethod
    def all(cls):
        for row in local.getDb().execute("SELECT room FROM events GROUP BY room"):
            yield cls()._init(row)

    @classmethod
    def count(cls):
        for row in local.getDb().execute("SELECT count(DISTINCT room) FROM events"):
            return row[0]

    @classmethod
    def filter(cls, day=None, track=None, event=None):
        filterdata, wherestm = _build_filter(day=day, track=track, event=event)

        query = """SELECT room FROM events
                    WHERE {}
                    GROUP BY room""".format(" AND ".join(wherestm))

        for row in local.getDb().execute(query, filterdata):
            yield cls()._init(row)


class Track:
    def _init(self, data):
        self.name = data['track']
        self.room = list(Room.filter(track=self))
        self.date = list(Day.filter(track=self))
        return self

    def __repr__(self):
        return "<{} {!r} in {!r} @ {}>".format(
            self.__class__.__name__, self.name, self.room, self.date)

    def __str__(self):
        return self.name or "- no track -"

    @classmethod
    def all(cls):
        for row in local.getDb().execute(
                "SELECT track, room, date FROM events WHERE track != '' GROUP BY track ORDER BY date, track"):
            yield cls()._init(row)

    @classmethod
    def count(cls):
        for row in local.getDb().execute("SELECT count(DISTINCT track) FROM events"):
            return row[0]


class Person:
    def _init(self, data):
        self.id = data['id']
        self.name = data['name']
        self.organization = data['organization']
        self.thumbnail = data['thumbnail']
        self.bio = data['bio']
        self.url = data['url']
        return self

    def events(self):
        for row in local.getDb().execute(
                """SELECT e.* FROM events as e
                    LEFT JOIN event_person AS ep ON e.id = ep.event_id
                    WHERE ep.person_id = ?""", (self.id,)):
            yield Event()._init(row)

    def __repr__(self):
        return "<{} {!r}>".format(self.__class__.__name__, self.name)

    def __str__(self):
        return self.name

    @classmethod
    def all(cls):
        for row in local.getDb().execute(
                "SELECT * FROM persons"):
            yield cls()._init(row)

    @classmethod
    def by_event(cls, event_id):
        for row in local.getDb().execute(
                """SELECT * FROM persons as p
                    LEFT JOIN event_person AS ep ON p.id = ep.person_id
                    WHERE ep.event_id = ?""", (event_id,)):
            yield cls()._init(row)


class Link:
    def _init(self, event, data):
        self.event = event
        self.href = data['href']
        self.name = data['name']
        return self

    def __repr__(self):
        return "<{} for {!r} {!r} {!r}>".format(
            self.__class__.__name__, self.event, self.name, self.href)

    def __str__(self):
        return '<a href="{}">{}</a>'.format(self.href, self.name)


class DbUpdateWatcherObj(GObject.GObject):
    @GObject.Signal
    def update(self):
        ...


dbUpdateWatcher = DbUpdateWatcherObj()


class Event(GObject.GObject):
    @GObject.Signal
    def update(self):
        ...

    def _init(self, data):
        self.id = data['id']
        self.date = data['date']
        self.start = data['start']
        self.end = data['end']
        self.room = data['room']
        self.slug = data['slug']
        self.title = data['title']
        self.subtitle = data['subtitle'] or ""
        self.track = data['track']
        self.evtype = data['type']
        self.abstract = data['abstract'] or ""
        self.description = data['description'] or ""
        self.starred = data['starred'] == 1
        self.notified = data['notified'] == 1
        return self

    def set_star(self, starred):
        local.getDb().execute("UPDATE events SET starred=? WHERE id=?", (starred, self.id,))
        local.getDb().commit()
        self.starred = starred
        self.emit('update')
        dbUpdateWatcher.emit('update')

    def set_notified(self, notified):
        local.getDb().execute("UPDATE events SET notified=? WHERE id=?", (notified, self.id,))
        local.getDb().commit()
        self.notified = notified
        self.emit('update')
        dbUpdateWatcher.emit('update')

    def start_in_tz(self):
        return self._dt_in_tz(self.start)

    def end_in_tz(self):
        return self._dt_in_tz(self.end)

    def _dt_in_tz(self, dt):
        dt = dt.replace(tzinfo=UTC)
        dt = dt.astimezone(Settings.instance().get_timezone())
        return dt

    def persons(self):
        # print("event", self.id, "persons()")
        return Person.by_event(self.id)

    def links(self):
        for row in local.getDb().execute(
            """SELECT href, name FROM links
                WHERE event_id=?""", (self.id,)):
            yield Link()._init(self, row)

    def __repr__(self):
        return "<{} {!r} in {!r} @ {!r}>".format(self.__class__.__name__, self.title, self.room, self.start)

    def __str__(self):
        return self.title

    def get_conflicts(self):
        query = """SELECT * FROM events
                    WHERE starred = 1 AND id != ? AND date = ?
                    AND end > ? AND start < ?
                """
        for row in local.getDb().execute(query, (self.id, self.date, self.start, self.end)):
            yield Event()._init(row)

    @classmethod
    def all(cls):
        row = local.getDb().execute("SELECT count(id) as c FROM events").fetchone()
        yield 0 if row is None else row['c']

        for row in local.getDb().execute("SELECT * FROM events"):
            yield cls()._init(row)

    @classmethod
    def by_id(cls, id):
        row = local.getDb().execute("SELECT * FROM events WHERE id = ?", (id,)).fetchone()
        if row is None:
            raise ObjectNotFound("<{} #{}>".format(cls.__name__, id))
        return cls()._init(row)

    @classmethod
    def filter(cls,
               day: Optional[Union[Day, str]] = None,
               room: Optional[Room] = None,
               track: Optional[Track] = None,
               order_by: str = "start, room",
               **kwargs):
        filterdata, wherestm = _build_filter("events", day=day, room=room, track=track, **kwargs)

        query = """SELECT * FROM events
                    WHERE {}
                    ORDER BY {}""".format(" AND ".join(wherestm), order_by)

        resultcountquery = query.replace("SELECT *", "SELECT count(events.id) as c")
        row = local.getDb().execute(resultcountquery, filterdata).fetchone()
        yield 0 if row is None else row['c']

        for row in local.getDb().execute(query, filterdata):
            yield cls()._init(row)

    @classmethod
    def nextup(cls, minutes):
        """get events starting in next `minutes`"""
        minutes = min(int(minutes), 30)  # some sanity check
        now = datetime.datetime.now(UTC).replace(tzinfo=None).strftime("%Y-%m-%dT%H:%M:%S.000")
        # now = "now" # is "now" somehow broken?
        query = """SELECT * FROM events
                    WHERE starred=1 AND notified=0
                        AND datetime(start) >= datetime('{1}')
                        AND datetime(start) <= datetime('{1}', '+{0} minutes');"""
        for row in local.getDb().execute(query.format(minutes, now)):
            yield cls()._init(row)

    @classmethod
    def search(cls, term, starred=False, order_by=None, **filters):
        """Search events for 'term'"""
        if starred:
            filters["starred"] = 1
        filterdata, wherestm = _build_filter("events", **filters)

        if term == "" and len(wherestm) == 0:
            yield 0
            return

        if term != "":
            order = ['rank']
            if order_by:
                order = [order_by, 'rank']

            wherestm.append("fts_event = ?")
            filterdata.append(term)

            query = """SELECT events.* FROM fts_event
                        JOIN events ON events.id = fts_event.event_id
                        WHERE {}
                        GROUP BY events.id
                        ORDER BY {}"""
        else:
            order = ['start', 'room']
            if order_by:
                order = [order_by]
            query = """SELECT events.* FROM events
                        WHERE {}
                        ORDER BY {}"""

        query = query.format(
            " AND ".join(wherestm),
            ", ".join(order)
        )

        resultcountquery = query.replace("SELECT events.*", "SELECT count(events.id) as c")
        row = local.getDb().execute(resultcountquery, filterdata).fetchone()
        yield 0 if row is None else row['c']

        results = local.getDb().execute(query, filterdata)
        for row in results:
            yield cls()._init(row)


class RecentConf(GObject.Object):
    __gtype_name__ = "ConfyRecentConf"

    @GObject.Property(type=str, default="")
    def uri(self):
        return self._uri

    @GObject.Property(type=str, default="<unknown>")
    def title(self):
        return self._title

    def __init__(self, recentinfo: Gtk.RecentInfo):
        super().__init__()
        self.data = recentinfo.get_description()
        self._uri = recentinfo.get_uri()
        self._title = recentinfo.get_display_name()

    def __str__(self):
        return f"{self._title} : {self._uri}"



# widgets/loadingoverlay.py
#
# Copyright 2020-2025 Fabio Comuni, et al.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from gettext import gettext as _

from gi.repository import Gtk
from gi.repository import Adw


def LoadingOverlay(cancellable, label=None):
    """
    This used to be a Class, but, again, Adw.Toast is final.
    Is a Toast that cancell the `cancellable` clicking on a button.
    """
    toast = Adw.Toast()
    toast.set_timeout(0)

    label = label or _("Loading...")

    adw_1_2 = Adw.get_major_version() > 1 or Adw.get_minor_version() > 1

    if adw_1_2:
        # this need libadwaita >=1.2
        box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=20)
        spinner = Gtk.Spinner.new()
        spinner.start()
        label = Gtk.Label(label=label)
        box.append(spinner)
        box.append(label)
        toast.set_custom_title(box)
    else:
        toast.set_title(label)

    toast.set_button_label(_("cancel"))

    cancellable.connect("done", lambda *_: toast.dismiss())
    cancellable.connect("error", lambda *_: toast.dismiss())
    if adw_1_2:
        # this need libadwaita 1.2
        toast.connect("button-clicked", lambda *_: cancellable.cancel())
    else:
        # for older adw, dismissing the toast will cancell the operation
        toast.connect("dismissed", lambda *_: cancellable.cancel())
    return toast



# preferences.py
#
# Copyright 2020-2025 Fabio Comuni, et al.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from gettext import gettext as _

from gi.repository import GObject
from gi.repository import Gio
from gi.repository import Gtk
from gi.repository import Adw

from .config import APP_NAME  # type:ignore # config is generated at compile time
from .settings import Settings, CacheDuration
from . import local


def format_size(num):
    for unit in ['B', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB', 'ZiB']:
        if abs(num) < 1024.0:
            return "%3.1f%s" % (num, unit)
        num /= 1024.0
    return "%.1f%s" % (num, 'Yi')


class SettingsCacheDurationRow(Adw.ComboRow):
    __gtype_name__ = "ConfySettingsCacheDurationRow"
    setting_key: str = GObject.Property(type=str, default="")

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.set_model(CacheDuration.as_model())
        self.connect("notify::setting-key", self._setup)

    def _setup(self, *args):
        Settings.instance().bind(self.setting_key, self, 'selected', Gio.SettingsBindFlags.DEFAULT)


@Gtk.Template(resource_path="/net/kirgroup/confy/preferences.ui")
class PreferencesWindow(Adw.PreferencesWindow):
    __gtype_name__ = "ConfyPreferencesWindow"

    @GObject.Property(type=str, default="", flags=GObject.ParamFlags.READABLE)
    def cache_space_label(self):
        cache_size = local.get_cache_size()
        occupied_space = format_size(cache_size)
        return _("Confy has {} of cached data").format(occupied_space)

    @Gtk.Template.Callback()
    def _delete_cached_files(self, btn, *_):
        local.clear_cache()
        self.notify("cache-space-label")

    @Gtk.Template.Callback()
    def _delete_recents(self, btn, *_):
        recentmanager = Gtk.RecentManager.get_default()
        for item in recentmanager.get_items():
            if item.has_application(APP_NAME):
                recentmanager.remove_item(item.get_uri())




# fetcher.py
#
# Copyright 2020-2022 Fabio Comuni, et al.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from threading import Thread
import os
import tempfile
import urllib.request

from gi.repository import GLib
from gi.repository import GObject


class DownloadCancelled(Exception):
    pass


class Fetcher(GObject.GObject):
    """ Fetch source to dest, asyncly.

    property
        running (bool, r) download is running
        fraction (floar, r) download progress 0.0 - 1.0

    signals:
        error (message:str)
        done (local_file_path:str)
    """

    @GObject.Signal(arg_types=(str,))
    def error(self, message):
        ...

    @GObject.Signal(arg_types=(str,))
    def done(self, local_file_path):
        ...

    @GObject.Property(type=bool, default=False)
    def running(self):
        return self._run

    @GObject.Property(type=float)
    def fraction(self):
        return self._fraction

    def __init__(self, source, dest=None, cbk=None):
        """ download `source` to `dest`
            `source`: uri
            `dest`: path. if None, then temp
            `cbk` : function called when done: `fn(local_file_path)`

            `cbk` is called before firing `done` signal. If any exception
            is raised by `cbk`, `error` signal is fired, instead.
        """
        super().__init__()

        self._run = True
        self._fraction = 0.0
        self._cbk = cbk

        if dest is None:
            _, dest = tempfile.mkstemp()
        Thread(target=self._fetch, args=(source, dest)).start()

    def _fetch(self, source, dest):
        try:
            with urllib.request.urlopen(source, timeout=20) as fsrc, open(dest, "wb") as fdst:
                total_size = int(fsrc.info().get('Content-Length', 0))
                block_size = 8192
                dwl_size = 0
                # python in runtime is old :)
                # while chunk := fsrc.read(block_size):
                chunk = fsrc.read(block_size)
                while chunk:
                    if not self._run:
                        raise DownloadCancelled("Download cacelled")
                    fdst.write(chunk)
                    if total_size > 0:
                        dwl_size += block_size
                        self._fraction = min(dwl_size, total_size) / total_size
                        GLib.idle_add(self.notify, "fraction")
                    chunk = fsrc.read(block_size)
        except Exception as e:
            if os.path.exists(dest):
                os.unlink(dest)
            import traceback
            traceback.print_exc()

            GLib.idle_add(self.emit, "error", str(e))
        else:
            GLib.idle_add(self._done, dest)

    def _done(self, dest):
        if self._cbk is not None:
            try:
                self._cbk(self, dest)
            except Exception as e:
                import traceback
                traceback.print_exc()

                GLib.idle_add(self.emit, "error", str(e))
                return
        GLib.idle_add(self.emit, "done", dest)

    def cancel(self):
        self._run = False
        self.notify("running")



# mainwindow.py
#
# Copyright 2020-2024 Fabio Comuni, et al.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from gettext import gettext as _

from gi.repository import GObject
from gi.repository import Gtk
from gi.repository import Adw


class ToggleSidebarButton(Gtk.ToggleButton):
    __gtype_name__ = "ConfyToggleSidebarButton"

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.connect("realize", self._on)

    def _on(self, *args):
        # find Adw.OverlaySplitView
        parent = self
        while parent.get_parent() is not None:
            parent = parent.get_parent()
            if parent.__class__ == Adw.OverlaySplitView:
                parent.bind_property("collapsed", self, "visible", GObject.BindingFlags.SYNC_CREATE)
                parent.bind_property("show-sidebar", self, "active", GObject.BindingFlags.BIDIRECTIONAL)



# widgets/searchbaroverlay.py
#
# Copyright 2020-2025 Fabio Comuni, et al.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from gi.repository import GLib
from gi.repository import GObject
from gi.repository import Gtk


@Gtk.Template(resource_path="/net/kirgroup/confy/widgets/searchbar.ui")
class SearchBarOverlay(Gtk.Box):
    """ Ora, mi piacerebbe sapere perchè cazzo Gtk.SearchBox è final,
        che chiaramente significa che non è subclassabile, solo che pare che
        posso farlo (almeno in PyGObject), ma scazza con il template.
        In pratica me lo costruisce, ma poi sparisce il child.
        Quindi sono costretto a wrapparlo in un elemento che posso subclassare.
        Ma dato che Gtk.Widget non ha child, la cosa più veloce è usare un Box.
        Così adesso il Box deve calcolare la dimensione dell'unico child che ha,
        la stracazzo di Gtk.SearchBox.
        E siccome questa doveva essere una subclass di Gtk.SearchBar, mi aspetto che
        si comporti come una Gtk.SearchBar e quindi sono costretto a riportare
        le properties e i metodi che uso nel codice.
        Bella merda.
    """
    __gtype_name__ = "ConfySearchBarOverlay"

    bar = Gtk.Template.Child()  # type: ignore
    entry = Gtk.Template.Child()  # type: ignore
    is_starred = Gtk.Template.Child()  # type: ignore

    @GObject.Signal(name="changed")
    def changed(self):
        ...

    _search_mode_enabled = False
    _show_close_button = False
    _show_advanced = True

    @GObject.Property(type=bool, default=False)
    def search_mode_enabled(self):
        return self._search_mode_enabled

    @search_mode_enabled.setter
    def search_mode_enabled_setter(self, value):
        self._search_mode_enabled = value

    @GObject.Property(type=bool, default=True)
    def show_close_button(self):
        return self._show_close_button

    @show_close_button.setter
    def show_close_button_setter(self, value):
        self._show_close_button = value

    @GObject.Property(type=bool, default=True)
    def show_advanced(self):
        return self._show_advanced

    @show_advanced.setter
    def show_advanced_setter(self, value):
        self._show_advanced = value

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.bind_property('search-mode-enabled', self.bar, 'search-mode-enabled', GObject.BindingFlags.BIDIRECTIONAL)
        self.bind_property('show-close-button', self.bar, 'show-close-button', GObject.BindingFlags.BIDIRECTIONAL)
        self.bar.connect('notify::search-mode-enabled', self._on_search_mode_changed)

    def _on_search_mode_changed(self, *args):
        if self.bar.get_search_mode():
            self.entry.set_text("")
            self.is_starred.set_active(False)
            self.entry.grab_focus()

    def set_search_mode(self, value):
        self.bar.set_search_mode(value)

    def grab_focus(self):
        GLib.idle_add(self.entry.grab_focus)

    def is_searching(self):
        return self.bar.get_search_mode() and (self.entry.get_text() != "" or self.is_starred.get_active())

    @Gtk.Template.Callback()
    def send_changed(self, *_):
        self.changed.emit()



# __init__.py
#
# Copyright 2020-2022 Fabio Comuni, et al.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
from ..fetcher import Fetcher
from .exceptions import InvalidFormatException
from .pentabarf import import_pentabarf
from .ics import import_ics
from gettext import gettext as _


def update_schedule(url: str) -> Fetcher:
    def _do_load(sender, local_filename: str):
        print("local conf file:", local_filename)
        with open(local_filename, "r") as f:
            content = f.read()
        if content.strip().startswith("<"):
            import_pentabarf(content, url)
        elif content.strip().startswith("BEGIN:VCALENDAR"):
            import_ics(content, url)
        else:
            raise InvalidFormatException(_("Unknown schedule format"))
        os.remove(local_filename)

    return Fetcher(url, cbk=_do_load)



# widgets/navbar.py
#
# Copyright 2020-2025 Fabio Comuni, et al.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from typing import Generator
from dataclasses import dataclass

from gi.repository import GObject
from gi.repository import Gtk


@Gtk.Template(resource_path="/net/kirgroup/confy/widgets/navbarlistitem.ui")
class NavbarListItem(Gtk.ListBoxRow):
    __gtype_name__ = "ConfyNavbarListItem"
    icon: str = GObject.Property(type=str, default="")
    label: str = GObject.Property(type=str, default="")
    group: int = GObject.Property(type=int, default=0)


@Gtk.Template(resource_path="/net/kirgroup/confy/widgets/navbar.ui")
class Navbar(Gtk.ListBox):
    __gtype_name__ = "ConfyNavbar"

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        # Just because I'm lazy:
        # set all chidldred of 'self' as self attribute
        # using template name (builder id)
        # this way I can add pages just in template and I'm done
        c = self.get_first_child()
        while c:
            name = c.get_buildable_id()
            if name:
                setattr(self, name, c)
            c = c.get_next_sibling()

    @GObject.Signal
    def page_changed(self):
        ...




# widgets/weblinkactionrow.py
#
# Copyright 2020-2025 Fabio Comuni, et al.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from os import path
from urllib.parse import urlparse
import html

from gi.repository import Gio
from gi.repository import Gtk
from gi.repository import Adw

from ..utils import clean_markup


class WebLinkActionRow(Adw.ActionRow):
    def __init__(self, title, url, icontype=None, **kwargs):
        opts = dict(activatable=True, selectable=False)
        opts.update(kwargs)
        super().__init__(**opts)
        self.url = url
        if title is None:
            title = path.basename(url)
        self.set_title(clean_markup(html.unescape(title)))
        self.set_subtitle(url)

        # if icontype is None:
        icontype = "text/html"
        parsedurl = urlparse(url)
        if parsedurl.path:
            ftype, _ = Gio.content_type_guess(parsedurl.path)
            fmaj, fmin = ftype.split("/")
            if fmaj in ('text', 'video', 'audio', 'image'):
                icontype = ftype
            if fmin == 'pdf' or 'document' in fmin:
                icontype = ftype

        self.content_type = icontype
        ficon = Gio.content_type_get_icon(icontype)

        image = Gtk.Image.new_from_gicon(ficon)
        image.set_icon_size(Gtk.IconSize.LARGE)
        self.add_prefix(image)


def weblinkactionrowactivated(listbox, actionrow):
    appinfo = Gio.app_info_get_default_for_type(actionrow.content_type, True)
    if appinfo is not None:
        appinfo.launch_uris([actionrow.url], None)
    else:
        Gio.app_info_launch_default_for_uri(actionrow.url, None)



# widgets/starbutton.py
#
# Copyright 2020-2023 Fabio Comuni, et al.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from gi.repository import GObject
from gi.repository import Gtk


@Gtk.Template(resource_path="/net/kirgroup/confy/widgets/starbutton.ui")
class StarButton(Gtk.Button):
    __gtype_name__ = "ConfyStarButton"

    _is_starred = False

    @GObject.Property(type=bool, default=True)
    def is_starred(self):
        return self._is_starred

    @is_starred.setter
    def is_starred_setter(self, v):
        self._is_starred = v
        self.notify("icon-name")

    @GObject.Property(type=str, default="non-starred-symbolic")
    def icon_name(self):
        return "starred-symbolic" if self._is_starred else "non-starred-symbolic"



# info.py
#
# Copyright 2020-2024 Fabio Comuni, et al.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from typing import Optional

from gi.repository import GObject
from gi.repository import Gio
from gi.repository import Gdk
from gi.repository import Gtk
from gi.repository import Adw

from .. import models
from ..config import FMT_DAY  # type:ignore # config is generated at compile time
from ..widgets import WebLinkActionRow, weblinkactionrowactivated
from ..utils import clean_markup


@Gtk.Template(resource_path="/net/kirgroup/confy/pages/info.ui")
class PageInfo(Adw.NavigationPage):
    __gtype_name__ = "ConfyPageInfo"

    conf: Optional[models.Conference] = None
    meta: Optional[models.Meta] = None

    statuspage = Gtk.Template.Child()  # type: ignore
    linkbox = Gtk.Template.Child()  # type: ignore

    @GObject.Property(type=str, default="")
    def description(self):
        desc = self.meta.venue or ""
        if self.meta.venue and self.meta.city:
            desc += "\n"
        desc += self.meta.city or ""
        if desc != "":
            desc += "\n\n"
        desc += self.meta.start.strftime(FMT_DAY)
        if self.meta.start != self.meta.end:
            desc += "\n" + self.meta.end.strftime(FMT_DAY)
        return clean_markup(desc)

    def __init__(self, win, conf, **kwargs):
        super().__init__(**kwargs)
        self.conf = conf
        self.update()
        win.connect("conf-updated", self.update)

    def _set_logo(self, fetcher, logofile):
        self.statuspage.set_paintable(
            Gdk.Texture.new_from_file(
                Gio.File.new_for_path(logofile)
            )
        )

    def update(self, *args):
        self.meta = self.conf.get_meta()
        if self.meta and self.meta.title:
            print("INFO set title to", self.meta.title)
            self.set_title(clean_markup(self.meta.title))

        self.statuspage.set_icon_name('net.kirgroup.confy-symbolic')
        self.conf.get_logo_file(cbk=self._set_logo)

        w = self.linkbox.get_last_child()
        while w:
            self.linkbox.remove(w)
            w = self.linkbox.get_last_child()

        links = self.conf.metadata.get("links", [])
        self.linkbox.set_visible(len(links) > 0)
        for link in links:
            row = WebLinkActionRow(link['title'], link['url'])
            self.linkbox.append(row)

        self.notify("description")

    @Gtk.Template.Callback()
    def on_linkbox_row_activated(self, listbox, row):
        weblinkactionrowactivated(listbox, row)



# info.py
#
# Copyright 2020-2024 Fabio Comuni, et al.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from typing import Optional

from gi.repository import GObject
from gi.repository import Gtk
from gi.repository import Adw

from .events import PageEvents
from .. import models
from ..config import FMT_DAY  # type:ignore # config is generated at compile time
from ..utils import clean_markup


@Gtk.Template(resource_path="/net/kirgroup/confy/pages/list.ui")
class PageList(Adw.NavigationPage):
    __gtype_name__ = "ConfyPageList"

    conf: Optional[models.Conference] = None
    win: Optional[Adw.ApplicationWindow] = None
    stack: Gtk.Stack = Gtk.Template.Child()  # type: ignore
    listbox: Gtk.ListBox = Gtk.Template.Child()  # type: ignore

    # a quanto pare non posso attaccare nessun tipo di userdata a un ActionRow
    # quindi per sapere a quale oggetto si riferisce l'ActionRow attivato
    # mi tocca tenere un bel dizionario. E' brutto? Si. Frega qualcosa? Decisamente no.
    # La vecchia versione teneva la lista corrente degli oggetti, e ricavava il
    # selezionato da 'row.get_index()' nell' handler di 'row-activated'.
    # E' meglio o peggio? boh.
    _widget_data: dict[str, models.Event] = {}
    _value: Optional[models.Event] = None

    def get_value(self):
        return self._value

    _model = ""

    @GObject.Property(type=str, default="")
    def model(self):
        return self._model

    @model.setter
    def model_setter(self, v):
        if v not in ["days", "tracks", "rooms"]:
            raise ValueError(f"Model {v} is not valid for PageList.")
        self._model = v

    def get_model(self):
        return self._model

    def __init__(self, win, conf, **kwargs):
        super().__init__(**kwargs)
        self.conf = conf
        self.win = win
        win.connect("conf-updated", self.update)
        self.update()

    def update(self, *_):
        count, data = self.get_objects()

        # empty listbox
        self._widget_data = {}
        self.listbox.remove_all()

        if count == 0:
            self.show_placeholder()
        else:
            self.stack.set_visible_child_name('list')

        # fill listbox
        for idx, obj in enumerate(data):
            row = self.build_row(obj)
            self._widget_data[row] = obj
            self.listbox.append(row)

    def build_row(self, obj):
        """
        Return a widget for data in current listbox
        """

        title = str(obj)
        subtitle = ""
        if self._model == "days":
            title = obj.date.strftime(FMT_DAY)
        elif self._model == "tracks":
            if title == "":
                title = "- no track -"
            subtitle = ", ".join(str(r) for r in obj.room)

        row = Adw.ActionRow(
            activatable=True,
            selectable=False,
            title=clean_markup(title),
            subtitle=clean_markup(subtitle)
        )
        icon = Gtk.Image.new_from_icon_name('go-next-symbolic')
        row.add_suffix(icon)

        return row

    def get_objects(self):
        if self._model == "days":
            return models.Day.count(), models.Day.all()
        elif self._model == "tracks":
            return models.Track.count(), models.Track.all()
        elif self._model == "rooms":
            return models.Room.count(), models.Room.all()
        else:
            return 0, []

    @Gtk.Template.Callback()
    def on_litstbox_activated(self, listbox, row):
        self._value = value = self._widget_data[row]
        page = PageEvents(value, model=self._model)
        self.get_parent().push(page)

    def show_placeholder(self):
        self.stack.set_visible_child_name('list')



import datetime
from gettext import gettext as _
from gi.repository import Gio
from gi.repository import GObject
from gi.repository import Gtk
from gi.repository import Adw

from . import local
from . import models
from .networkconnectionmonitor import NetworkConnectionMonitor
from .widgets import InputDialog
from .widgets import EventEditDialog
from .widgets import LoadingOverlay


@Gtk.Template(resource_path="/net/kirgroup/confy/openwindow.ui")
class OpenWindow(Adw.ApplicationWindow):
    __gtype_name__ = "ConfyOpenWindow"

    overlay = Gtk.Template.Child()  # type: ignore
    searchbar = Gtk.Template.Child()  # type: ignore
    stack = Gtk.Template.Child()  # type: ignore
    empty_placeholder = Gtk.Template.Child()  # type: ignore
    nomatch_placeholder = Gtk.Template.Child()  # type: ignore
    content = Gtk.Template.Child()  # type: ignore

    heading_next = Gtk.Template.Child()  # type: ignore
    listbox_next = Gtk.Template.Child()  # type: ignore
    heading_past = Gtk.Template.Child()  # type: ignore
    listbox_past = Gtk.Template.Child()  # type: ignore

    _in_edit = False

    @GObject.Property(type=bool, default=False)
    def in_edit(self):
        return self._in_edit

    @in_edit.setter
    def in_edit_setter(self, value):
        self._in_edit = value

    def __init__(self, **kwargs):
        self.nm = NetworkConnectionMonitor()
        self.data = []

        super().__init__(**kwargs)

        self.stack.set_visible_child(self.empty_placeholder)

        fetcher = local.open_menu(self.nm.props.isconnected)
        if fetcher is not None:
            self.loading_show(fetcher)
            fetcher.connect("done", lambda *_: self.update_list())
            fetcher.connect("error", lambda s, e: self.notification_show(_("<b>Error loading events list:</b>\n{}").format(e)))
        else:
            self.update_list()

        self._add_action("add-event", None, ["<Primary>a"], self.open_custom_url)
        self._add_action("refresh-events", None, ["<Primary>r"], self.update_schedules)
        # TODO: disable action if there is no user entris in menu
        self.edit_meta_action = self._add_action("edit-meta", None, None, lambda *_: self.set_in_edit(not self.props.in_edit))
        self._add_action("search", None, ["<Primary>f"], lambda *_: self.searchbar.set_search_mode(True))

        # can't do this because: Warning: ../glib/gobject/gbinding.c:271: Unable to convert a value of type PyObject to a value of type gboolean
        # self.nm.bind_property("isconnected", refresh_events_action, "enabled", 0 )
        self.nm.connect("notify", self.on_networkconnectionmonitor_notify)
        self.on_networkconnectionmonitor_notify()

    def _add_action(self, name, parameter_type, accels, activate_cbk):
        action = Gio.SimpleAction.new(name, parameter_type)
        action.connect("activate", activate_cbk)
        self.add_action(action)
        if accels is not None:
            self.get_application().set_accels_for_action(F"win.{name}", accels)
        return action

    def set_in_edit(self, value):
        self.props.in_edit = value

    @Gtk.Template.Callback()
    def on_do_search(self, *args):
        self.update_list()

    def on_networkconnectionmonitor_notify(self, *args):
        is_connected = self.nm.props.isconnected
        self.lookup_action("refresh-events").set_enabled(is_connected)
        self.lookup_action("add-event").set_enabled(is_connected)

    def update_schedules(self, *args):
        def _done(s, f):
            self.update_list()
            self.notification_show(_("<b>Events list updated</b>"))

        def _error(s, e):
            self.notification_show(_("<b>Error loading events list:</b>\n{}").format(e))

        fetcher = local.update_menu()
        self.loading_show(fetcher)
        fetcher.connect("done", _done)
        fetcher.connect("error", _error)

    def delete_conf(self, conf, cbk=None):
        def _message_response(dialog, response):
            if response == "delete":
                # delete from user menu
                local.delete_user_menu(conf.to_json())
                # delete from recent list
                Gtk.RecentManager.get_default().remove_item(conf.url)
                self.update_list()
                if cbk is not None:
                    cbk(conf)
            dialog.destroy()
        message = Adw.MessageDialog(transient_for=self)
        message.props.heading = _('Delete event "{}"?').format(conf.title)
        message.props.body = _("Event will be removed from event list and related data will be deleted")
        message.add_response("cancel", _("Cancel"))
        message.add_response("delete", _("Delete"))
        message.set_response_appearance("delete", Adw.ResponseAppearance.DESTRUCTIVE)
        message.set_default_response("cancel")
        message.set_close_response("cancel")
        message.connect("response", _message_response)
        message.present()

    def edit_conf(self, conf, ok_cbk=None, del_cbk=None, can_cancel=True, can_delete=False):
        def _save(dialog):
            conf = d.get_conf()
            local.update_user_menu(conf.to_json())
            if conf.has_cache():
                with conf.get_meta() as m:
                    m.title = conf.title
                    m.start = conf.start
                    m.end = conf.end
            self.update_list()
            if ok_cbk is not None:
                ok_cbk(conf)
            d.close()

        def _delete(dialog):
            self.delete_conf(conf, lambda *_: d.close())

        if conf is None:
            return
        d = EventEditDialog(self, conf, can_delete=can_delete, can_cancel=can_cancel)
        d.connect("save", _save)
        d.connect("delete", _delete)
        d.connect("cancel", lambda *args: d.close())
        d.present(self)

    def open_custom_url(self, *args):
        def _cbk(conf):
            self.notification_show(_("<b>New event:</b> {}").format(conf.title))

        def _error(s, e):
            self.notification_show(_("<b>Error:</b> {}").format(e))

        def _add_menu_ckb(meta):
            conf = models.Conference.by_url(meta.url)
            self.edit_conf(conf, can_cancel=False, ok_cbk=_cbk)

        def _dialog_response(dialog, res):
            if res == InputDialog.OK:
                url = dialog.get_text()
                try:
                    fetcher = local.add_user_menu(url, _add_menu_ckb)
                    if fetcher is not None:
                        fetcher.connect("error", _error)
                except local.MenuItemAlreadyExistsException as e:
                    # edit existing obj
                    self.notification_show(_("Event already in list"))
                    conf = models.Conference()._init(e.obj)
                    self.edit_conf(conf, can_cancel=True)
                except Exception as e:
                    import traceback
                    traceback.print_exc()

                    self.notification_show(_("<b>Error:</b> {}").format(e))
                else:
                    if fetcher is not None:
                        self.loading_show(fetcher)
            dialog.destroy()

        dialog = InputDialog(self)
        dialog.connect("response", _dialog_response)
        dialog.present()

    def _empty_list(self, listbox):
        w = listbox.get_last_child()
        while w:
            listbox.remove(w)
            w = listbox.get_last_child()

    def update_list(self):
        sb = self.searchbar
        try:
            confs = models.Conference.all()

            # if searchbar is visible and search text is not "", filter confs by title and url
            if sb.is_searching():
                search = sb.entry.get_text().lower()
                confs = [conf for conf in confs
                         if search in conf.title.lower() or search in conf.url.lower()]
        except Exception as e:
            self.notification_show(_("<b>Error loading events list:</b>\n{}").format(e))

        self._empty_list(self.listbox_next)
        self._empty_list(self.listbox_past)

        count = len(confs)

        if count == 0:
            if self.searchbar.is_searching():
                self.stack.set_visible_child(self.nomatch_placeholder)
            else:
                self.stack.set_visible_child(self.empty_placeholder)
        else:
            self.stack.set_visible_child(self.content)

        confs_next = []
        confs_past = []

        for conf in confs:
            row = self.build_row(conf)
            if conf.end < datetime.date.today():
                confs_past.append(conf)
                self.listbox_past.append(row)
            else:
                confs_next.append(conf)
                self.listbox_next.append(row)

        show_next = len(confs_next) > 0
        show_past = len(confs_past) > 0
        self.listbox_next.set_visible(show_next)
        self.heading_next.set_visible(show_next)
        self.listbox_past.set_visible(show_past)
        self.heading_past.set_visible(show_past)

        self.data = [
            confs_next,
            confs_past
        ]

    def build_row(self, obj):
        row = Adw.ActionRow(activatable=True, selectable=False)
        row.set_title(obj.title)

        self.bind_property("in-edit", row, "activatable", GObject.BindingFlags.SYNC_CREATE | GObject.BindingFlags.INVERT_BOOLEAN)

        if obj.user:
            delete_button = Gtk.Button(
                icon_name="user-trash-symbolic",
                valign=Gtk.Align.CENTER,
            )
            delete_button.add_css_class("destructive-action")
            delete_button.connect("clicked", lambda *_: self.delete_conf(obj))
            self.bind_property("in-edit", delete_button, "visible", GObject.BindingFlags.SYNC_CREATE)
            row.add_suffix(delete_button)

            edit_button = Gtk.Button(
                icon_name="document-edit-symbolic",
                valign=Gtk.Align.CENTER,
            )
            edit_button.add_css_class("suggested-action")
            edit_button.connect("clicked", lambda *_: self.edit_conf(obj, can_delete=obj.user))
            self.bind_property("in-edit", edit_button, "visible", GObject.BindingFlags.SYNC_CREATE)
            row.add_suffix(edit_button)
        return row

    @Gtk.Template.Callback()
    def on_activated_next(self, listbox, actionrow):
        idx = actionrow.get_index()
        obj = self.data[0][idx]
        self.props.application.open_conf(obj)

    @Gtk.Template.Callback()
    def on_activated_past(self, listbox, actionrow):
        idx = actionrow.get_index()
        obj = self.data[1][idx]
        self.props.application.open_conf(obj)

    def loading_show(self, cancellable):
        t = LoadingOverlay(cancellable)
        self.overlay.add_toast(t)

    def notification_show(self, text):
        t = Adw.Toast()
        t.set_title(text)
        self.overlay.add_toast(t)



# pentabarf.py
#
# Copyright 2020-2025 Fabio Comuni, et al.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from typing import Optional

import datetime
import time
import sqlite3
import xml.etree.ElementTree as ET
from gettext import gettext as _

from dateutil.tz import UTC, tzlocal

from .exceptions import InvalidFormatException
from .. import local
from ..models import Meta
from ..utils import bind_list


def _local_to_utc_to_unset(dt):
    dt = dt.replace(tzinfo=tzlocal())
    dt = dt.astimezone(UTC)
    return dt.replace(tzinfo=None)


def _get_text(root: Optional[ET.Element], nodename: str) -> str:
    text = ""
    if root is not None:
        node = root.find(nodename)
        if node is not None and node.text is not None:
            text = node.text
    return text


def import_pentabarf(xmlstr: str, url: str):
    """Import data from Pentabarf XML

    As far I can tell, Pentabarf XML doesn't declare timezone.
    We will import dates in local timezone, then convert to UTC, then
    remove timezone info to store dates in db (because sqlite3 cries otherwise)
    """
    root = ET.fromstring(xmlstr)
    if root.tag != "schedule":
        raise InvalidFormatException(_("Invalid pentabarf format"))

    econference = root.find("conference")
    base_url = _get_text(econference, "base_url")
    with Meta() as m:
        m.url = url
        m.last_update = time.time()  # timestamp
        m.title = _get_text(econference, "title")
        m.start = _get_text(econference, "start")
        m.end = _get_text(econference, "end")
        m.venue = _get_text(econference, "venue")
        m.days = _get_text(econference, "days")
        m.city = _get_text(econference, "city")

    _db = local.getDb()

    found_event_ids = []
    found_person_ids = []

    for eday in root.iter('day'):
        date = eday.attrib['date']
        # found in sfscon. events nested in <room name=""> element
        # if we find 'day > room', we parse children events using
        # room name from room elements
        rooms = eday.findall("room")
        if rooms:
            for eroom in rooms:
                room = eroom.attrib['name']
                feids, fpids = _handle_events(_db, eroom, base_url, date, room)
                found_event_ids += feids
                found_person_ids += fpids
        else:
            # if we don't find 'day > room', we then parse as 'day > event'
            found_event_ids, found_person_ids = _handle_events(_db, eday, base_url, date)

    # cleanup removed events and persons
    _db.execute(f"""DELETE FROM persons WHERE id NOT IN ({bind_list(found_person_ids)})""", found_person_ids)
    _db.execute(f"""DELETE FROM events WHERE id NOT IN ({bind_list(found_event_ids)})""", found_event_ids)
    _db.execute(f"""DELETE FROM fts_event WHERE event_id NOT IN ({bind_list(found_event_ids)})""", found_event_ids)

    _db.commit()


def _handle_events(
        _db: sqlite3.Connection,
        root: ET.Element,
        base_url: str,
        date: str,
        room: Optional[str] = None) -> tuple[list, list]:

    found_event_ids = []
    found_person_ids = []

    for eevent in root.iter('event'):
        fulltextsearch = []
        # found in sfscon xml. event 'id' changes,
        # while unique_id not.
        # we use unique_id if is set
        if 'unique_id' in eevent.attrib:
            eventid = eevent.attrib['unique_id']
        else:
            eventid = eevent.attrib['id']
        found_event_ids.append(eventid)

        tstart = _get_text(eevent, 'start').strip()[:5]
        start = _local_to_utc_to_unset(
            datetime.datetime.strptime(date + " " + tstart, "%Y-%m-%d %H:%M")
        )
        end = _get_text(eevent, 'duration').split(":")
        end = start + datetime.timedelta(hours=int(end[0]), minutes=int(end[1]))
        evtdate = datetime.datetime.strptime(date, "%Y-%m-%d").date()
        if room is None:
            room = _get_text(eevent, 'room')
        slug = _get_text(eevent, 'slug')
        title = _get_text(eevent, 'title')
        subtitle = _get_text(eevent, 'subtitle')
        track = _get_text(eevent, 'track')
        evtype = _get_text(eevent, 'type')
        abstract = _get_text(eevent, 'abstract')
        description = _get_text(eevent, 'description')
        persons = []
        for e in eevent.iter('person'):
            if e.text is None:
                continue
            name = e.text
            id = e.attrib.get('id', name)
            org = e.attrib.get('organizzation', '')
            thumbnail = e.attrib.get('thumbnail', '')
            if thumbnail and thumbnail.startswith("/"):
                thumbnail = base_url + thumbnail
            bio = e.attrib.get('bio', '')
            url = e.attrib.get('url', '')
            if url and url.startswith("/"):
                url = base_url + url
            persons.append((
                id, name, org, thumbnail, bio, url
            ))
            found_person_ids.append(id)

        fulltextsearch += [s for s in [title, subtitle, abstract, description, room, track] if s is not None]
        fulltextsearch += [p[1] for p in persons]

        links = []
        for e in eevent.iter('link'):
            if e.text is None:
                continue
            name = e.text
            href = e.attrib.get('href', "")
            if not href:
                href = name
            if base_url and href.startswith("/"):
                href = base_url + href
            links.append((eventid, href, name))

        for e in eevent.iter('attachment'):
            if e.text is None:
                continue
            name = e.text
            if e.attrib.get('type', False):
                name = '{} ({})'.format(name, e.attrib['type'])
            href = e.attrib.get('href', "")
            if not href:
                href = name
            if base_url and href.startswith("/"):
                href = base_url + href
            links.append((eventid, href, name))

        _db.execute("""INSERT OR REPLACE INTO events
                        (id, date, start, end, room, slug, title, subtitle, track, type, abstract, description, starred)
                        VALUES (?,?,?,?,?,?,?,?,?,?,?,?, (SELECT starred FROM events WHERE id=?))""",
                    (eventid, evtdate, start, end, room, slug, title, subtitle, track, evtype, abstract, description, eventid))
        _db.executemany("INSERT OR REPLACE INTO persons (id, name, organization, thumbnail, bio, url) VALUES (?, ?, ?, ?, ?, ?)", persons)
        _db.executemany("""INSERT OR REPLACE INTO event_person (event_id, person_id)
                            VALUES (?, ?) ON CONFLICT DO NOTHING""",
                        [(eventid, p[0]) for p in persons])
        _db.executemany("""INSERT OR REPLACE INTO links
                            (event_id, href, name) VALUES (?,?,?)""", links)

        fulltextsearchstr = ' '.join(fulltextsearch)
        _db.execute("""INSERT OR REPLACE INTO fts_event (event_id, text) VALUES (?, ?)""", (eventid, fulltextsearchstr))

    return found_event_ids, found_person_ids
